﻿/*
 * Created by SharpDevelop.
 * User: jt
 * Date: 30-10-2010
 * Time: 19:21
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections;
using System.Windows.Forms;



namespace HovText
{
    /// <summary>
    /// A simple control that allows the user to select pretty much any valid hotkey combination
    /// </summary>
    public class HotKeyControl : TextBox
    {
        // These variables store the current hotkey and modifier(s)
        private Keys _hotkey = Keys.None;
        private bool shift;
        private bool control;
        private bool alt;
		private bool windows;
		private bool firstKeyPress = true;

        // ArrayLists used to enforce the use of proper modifiers.
        // Shift+A isn't a valid hotkey, for instance, as it would screw up when the user is typing.
        private ArrayList needNonShiftModifier = null;
        private ArrayList needNonAltGrModifier = null;

        private ContextMenu dummy = new ContextMenu();

        /// <summary>
        /// Used to make sure that there is no right-click menu available
        /// </summary>
        public override ContextMenu ContextMenu
        {
            get
            {
                return dummy;
            }
            set
            {
                base.ContextMenu = dummy;
            }
        }

        /// <summary>
        /// Forces the control to be non-multiline
        /// </summary>
        public override bool Multiline
        {
            get
            {
                return base.Multiline;
            }
            set
            {
                // Ignore what the user wants; force Multiline to false
                base.Multiline = false;
            }
        }

        /// <summary>
        /// Creates a new HotkeyControl
        /// </summary>
        public HotKeyControl()
        {
            this.ContextMenu = dummy; // Disable right-clicking
            this.Text = "None";

            // Handle events that occurs when keys are pressed
            this.KeyPress += new KeyPressEventHandler(HotkeyControl_KeyPress);
            this.KeyUp += new KeyEventHandler(HotkeyControl_KeyUp);
            this.KeyDown += new KeyEventHandler(HotkeyControl_KeyDown);

            // Fill the ArrayLists that contain all invalid hotkey combinations
            needNonShiftModifier = new ArrayList();
            needNonAltGrModifier = new ArrayList();
            PopulateModifierLists();
        }

        /// <summary>
        /// Populates the ArrayLists specifying disallowed hotkeys
        /// such as Shift+A, Ctrl+Alt+4 (would produce a dollar sign) etc
        /// </summary>
        private void PopulateModifierLists()
        {
            // Shift + 0 - 9, A - Z
            for (Keys k = Keys.D0; k <= Keys.Z; k++)
                needNonShiftModifier.Add((int)k);

            // Shift + Numpad keys
            for (Keys k = Keys.NumPad0; k <= Keys.NumPad9; k++)
                needNonShiftModifier.Add((int)k);

            // Shift + Misc (,;<./ etc)
            for (Keys k = Keys.Oem1; k <= Keys.OemBackslash; k++)
                needNonShiftModifier.Add((int)k);

            // Shift + Space, PgUp, PgDn, End, Home
            for (Keys k = Keys.Space; k <= Keys.Home; k++)
                needNonShiftModifier.Add((int)k);

            // Misc keys that we can't loop through
            needNonShiftModifier.Add((int)Keys.Insert);
            needNonShiftModifier.Add((int)Keys.Help);
            needNonShiftModifier.Add((int)Keys.Multiply);
            needNonShiftModifier.Add((int)Keys.Add);
            needNonShiftModifier.Add((int)Keys.Subtract);
            needNonShiftModifier.Add((int)Keys.Divide);
            needNonShiftModifier.Add((int)Keys.Decimal);
            needNonShiftModifier.Add((int)Keys.Return);
            needNonShiftModifier.Add((int)Keys.Escape);
            needNonShiftModifier.Add((int)Keys.NumLock);
            needNonShiftModifier.Add((int)Keys.Scroll);
            needNonShiftModifier.Add((int)Keys.Pause);

            // Ctrl+Alt + 0 - 9
            for (Keys k = Keys.D0; k <= Keys.D9; k++)
                needNonAltGrModifier.Add((int)k);
        }

        /// <summary>
        /// Resets this hotkey control to None
        /// </summary>
        public new void Clear()
        {
          this.alt=this.control=this.shift=this.windows=false;
          this.Key = Keys.None;
        }

        /// <summary>
        /// Fires when a key is pushed down. Here, we'll want to update the text in the box
        /// to notify the user what combination is currently pressed.
        /// </summary>
        void HotkeyControl_KeyDown(object sender, KeyEventArgs e)
        {
            if (firstKeyPress)
            {
            	windows = false;
            	firstKeyPress = false;
            }
            // Clear the current hotkey            
            if (e.KeyCode == Keys.Back || e.KeyCode == Keys.Delete)
            {
                ResetHotkey();
                return;
            } 
            if (e.KeyCode == Keys.LWin)
            	this.windows = true;
            else
            	this._hotkey = e.KeyCode;
            this.alt = e.Alt;
            this.control = e.Control;
            this.shift = e.Shift;
			Redraw();
        }

        /// <summary>
        /// Fires when all keys are released. If the current hotkey isn't valid, reset it.
        /// Otherwise, do nothing and keep the text and hotkey as it was.
        /// </summary>
        void HotkeyControl_KeyUp(object sender, KeyEventArgs e)
        {
            if (this._hotkey == Keys.None && Control.ModifierKeys == Keys.None)
            {
                ResetHotkey();
                return;
            }
			Redraw();
        }

        /// <summary>
        /// Prevents the letter/whatever entered to show up in the TextBox
        /// Without this, a "A" key press would appear as "aControl, Alt + A"
        /// </summary>
        void HotkeyControl_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        /// <summary>
        /// Handles some misc keys, such as Ctrl+Delete and Shift+Insert
        /// </summary>
        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            if (keyData == Keys.Delete || keyData == (Keys.Control | Keys.Delete))
            {
                ResetHotkey();
                return true;
            }

            if (keyData == (Keys.Shift | Keys.Insert)) // Paste
                return true; // Don't allow

            // Allow the rest
            return base.ProcessCmdKey(ref msg, keyData);
        }

        /// <summary>
        /// Clears the current hotkey and resets the TextBox
        /// </summary>
        public void ResetHotkey()
        {
            this._hotkey = Keys.None;
          this.alt=this.control=this.shift=this.windows=false;
            Redraw();
        }

        /// <summary>
        /// Used to get/set the hotkey (e.g. Keys.A)
        /// </summary>
        public Keys Key
        {
            get
            {
                return this._hotkey;
            }
            set
            {
                this._hotkey = value;
                Redraw(true);
            }
        }

       public bool AltKey {
        	get { return alt;}
        	set { alt = value; Redraw(true);}
        }
        public bool ControlKey {
        	get { return control;}
        	set { control = value; Redraw(true);}
        }
        public bool ShiftKey {
        	get { return shift; }
        	set { shift = value; Redraw(true);}
        }
        public bool WindowsKey {
        	get { return windows;}
        	set { windows= value; Redraw(true);}
        }
 
        /// <summary>
        /// Helper function
        /// </summary>
        private void Redraw()
        {
            Redraw(false);
        }

        /// <summary>
        /// Redraws the TextBox when necessary.
        /// </summary>
        /// <param name="bCalledProgramatically">Specifies whether this function was called by the Hotkey/HotkeyModifiers properties or by the user.</param>
        private void Redraw(bool bCalledProgramatically)
        {
            if (this._hotkey == Keys.RWin)
            {
				this._hotkey = Keys.None;            
            }
            
            Keys modifier = Keys.None;
            if (alt)
            	modifier |= Keys.Alt;
            if (shift)
            	modifier |= Keys.Shift;
            if (control)
            	modifier |= Keys.Control;

            // Only validate input if it comes from the user
            if (bCalledProgramatically == false)
            {
                // No modifier or shift only, AND a hotkey that needs another modifier
                if ((modifier == Keys.Shift || 
                     (modifier == Keys.None && !windows)) &&
                this.needNonShiftModifier.Contains((int)this._hotkey))
                {
                    if (modifier == Keys.None && !windows)
                    {
                        // Set Ctrl+Alt as the modifier unless Ctrl+Alt+<key> won't work...
                        if (needNonAltGrModifier.Contains((int)this._hotkey) == false)
                            modifier = Keys.Alt | Keys.Control;
                        else // ... in that case, use Shift+Alt instead.
                            modifier = Keys.Alt | Keys.Shift;
                    }
                    else if (!windows)
                    {
                        // User pressed Shift and an invalid key (e.g. a letter or a number),
                        // that needs another set of modifier keys
                        this._hotkey = Keys.None;
                        this.Text = modifier.ToString() + " + Invalid key";
                        return;
                    }
                }
                // Check all Ctrl+Alt keys
                if ((modifier == (Keys.Alt | Keys.Control)) &&
                    this.needNonAltGrModifier.Contains((int)this._hotkey))
                {
                    // Ctrl+Alt+4 etc won't work; reset hotkey and tell the user
                    this._hotkey = Keys.None;
                    this.Text = modifier.ToString() + " + Invalid key";
                    return;
                }
            }

            if (modifier == Keys.None && !windows)
            {
	            this.Text = "None";
	            return;
            }

            // I have no idea why this is needed, but it is. Without this code, pressing only Ctrl
            // will show up as "Control + ControlKey", etc.
            if (this._hotkey == Keys.Menu /* Alt */ || this._hotkey == Keys.ShiftKey || this._hotkey == Keys.ControlKey)
                this._hotkey = Keys.None;

            if (modifier == Keys.None)
            	this.Text = "Win + " + this._hotkey.ToString();
            else if (windows)
            	this.Text = "Win + " + modifier.ToString() + " + " + this._hotkey.ToString();
            else
	            this.Text = modifier.ToString() + " + " + this._hotkey.ToString();
            	
        }
    }
}